import com.holub.asynch.Condition;
import java.util.Date;

/** A generic deamon thread that uses the wait-for-start
 *	strategy. Runs until interrupted, but guarantees
 *  that the operation that you provide in the constructor will
 *  not be terminated by a premature JVM shut down.
 *  @param operation The operation to perform on the thread
 *	@param delay	 The run() method of this object is called
 *					 at the end of the iteration. It can
 *					 terminate the daemon by calling
 *					 <code>Thread.currentThread().interrupt()</code>.
 */

final class Wait_for_start extends Thread
{
	Runnable operation;
	Runnable delay;

	public Wait_for_start( Runnable operation, Runnable delay )
	{	setDaemon(true);
		this.operation = operation;
		this.delay	   = delay;
	}

	public void run()
	{
	  try
	  {
		while( !isInterrupted() )
		{
			final Condition lock_is_running = new Condition( false ); //#lock_is_running

			Thread lock = new Thread() //#create
						  {	synchronized public void run()
							{	lock_is_running.set_true(); //#release
								try
								{	wait();
								}
								catch(InterruptedException e)
								{	// eat it
								}
							}
						  };
			lock.start();						//#start
			lock_is_running.wait_for_true();	//#wait

			operation.run();

			// The deamon will not be killed by the JVM
			// until the following notify is executed.

			synchronized( lock )
			{	lock.notify(); 	// Allow the lock thread to terminate. //#notify
			}

			delay.run();
		}
	  }
	  catch( InterruptedException e )
	  {	// ignore it, we terminate this thread by interrupting it.
	  }
	}
	//----------------------------------------------------------------
	public static class Test
	{	public static void main( String[] args )
											throws InterruptedException
		{	new Wait_for_start
				(	new Runnable()	// operation
					{	public void run()
						{	System.out.println( new Date() );
						}
					},
					new Runnable()	// delay
					{	public void run()
						{	try
							{	Thread.currentThread().sleep(1000);
							}
							catch(InterruptedException e){}
						}
					}
				).start();

			Thread.currentThread().sleep(5000);
		}
	}
}
